Decode

Trait Decode 

pub trait Decode: Sized {
    // Required method
    fn decode(bytes: &[u8]) -> Option<Self>;
}
Decodes a point from its compressed or uncompressed representation

Required Methods

fn decode(bytes: &[u8]) -> Option<Self>

Decodes a point from its compressed or uncompressed representation

Dyn Compatibility

This trait is not dyn compatible.

In older versions of Rust, dyn compatibility was called "object safety".

Implementors